@@ -4,12 +4,14 @@
 
 use std::{
     borrow::{Borrow, BorrowMut},
-    ops::{Deref, DerefMut},
+    collections::BTreeMap,
+    ops::{ControlFlow, Deref, DerefMut},
 };
 
 use serde::{Deserialize, Serialize};
 
 pub mod parser;
+pub mod writer;
 
 /// Node in a CRC tree
 #[derive(Debug, Copy, Clone, Serialize, Deserialize, PartialEq, Eq)]
@@ -49,3 +51,19 @@ impl<D> DerefMut for CRCTreeNode<D> {
         &mut self.data
     }
 }
+
+/// Datastructure to hold a CRC tree.
+///
+/// Within the file, the trees are sorted by CRC value and organised in
+/// binary tree. This is not necessarily the same as the Rust B-Tree, but
+/// the ordering is good enough for what we need.
+pub type CRCTree<T> = BTreeMap<u32, T>;
+
+/// A trait to visit a CRC tree from a reader
+pub trait CRCTreeVisitor<T> {
+    /// The type of data to return on a premature break
+    type Break;
+
+    /// Called once for every
+    fn visit(&mut self, crc: u32, data: T) -> ControlFlow<Self::Break>;
+}
